/*
 * Copyright (c) 2009-2012 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * MEAS_TSYS02D_Main.c: Console Application for Testing the TSYS02D
 *
 * This application configures UART 16550 to baud rate 9600.
 * PS7 UART (Zynq) is not initialized by this application, since
 * bootrom/bsp configures it to baud rate 115200
 *
 * ------------------------------------------------
 * | UART TYPE   BAUD RATE                        |
 * ------------------------------------------------
 *   uartns550   9600
 *   uartlite    Configurable only in HW design
 *   ps7_uart    115200 (configured by bootrom/bsp)
 */

#include <stdio.h>
#include <unistd.h>
#include "platform.h"
#include "xparameters.h"
//#include "sleep.h"
#include "tsys02d.h"

void tsys02d_main_menu(void);

int main()
{

    char key_input;
    int i;
    tsys02d_status stat;
    float temperature;

    //Initialize the UART
    init_platform();

    //printf("Hello World\n");

    // Set the AXI address of the IIC core
    tsys02d_init(XPAR_AXI_IIC_JC_BASEADDR);

    // Display the main menu
    tsys02d_main_menu();

    // Infinite loop
    while(1){

        // Get keyboard input
        read(1, (char*)&key_input, 1);

        if(key_input == '1'){         //If the '1' key is pressed

            // Send the reset command to the TSYS02D
            printf("\n");
            printf("Resetting TSYS02D...\n");
            stat = tsys02d_reset();

            // Display the status returned from the reset operation
            printf("TSYS02D Reset Complete with status: ");
            if(stat==tsys02d_status_ok)
                printf("Ok.\n");
            if(stat==tsys02d_status_i2c_transfer_error)
                printf("Transfer Error.\n");

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys02d_main_menu();

        }else if(key_input == '2'){         // If the '2' key is pressed

            // Read the serial number from the TSYS02D
            printf("\n");
            printf("Reading Serial Number...\n");
            stat = tsys02d_read_serial_number();

            // Display status returned from read_prom operation
            // and display prom values if successful
            printf("Read Serial Number Complete with status: ");
            if(stat==tsys02d_status_ok){
                printf("Ok.\n");
                printf("\n");
                printf("_________________________________________________\n");
                printf("| Serial Byte # |    Base 10    |     (Hex)     |\n");
                printf("|---------------+---------------+---------------|\n");
                for(i=0;i<8;i++){
                    printf("|\t%d\t|     %5d\t|      0x%2X\t|\n",i,(unsigned int)tsys02d_serial_number[i],(unsigned int)tsys02d_serial_number[i]);
                }
            }else if(stat==tsys02d_status_i2c_transfer_error){
                printf("Transfer Error.\n");
            }else if(stat==tsys02d_status_crc_error){
            	printf("CRC Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys02d_main_menu();

        }else if(key_input == '3'){         // If the '3' key is pressed

        	// Read one temperature value
            printf("\n");
            printf("Reading Temperature Value...\n");
            stat = tsys02d_read_temperature(&temperature);

            // Display the status returned from the read_temperature
            // operation and display the temperature if successful
            printf("Temperature Read Complete with status: ");
            if(stat==tsys02d_status_ok){
                printf("Ok.\n");
                printf("Temperature : %5.2f%cC",temperature,248);
            }else if(stat==tsys02d_status_i2c_transfer_error){
                printf("Transfer Error.");
            }else if(stat==tsys02d_status_crc_error){
            	printf("CRC Error.");
            }
            printf("\n");

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys02d_main_menu();

        }else if(key_input == '4'){         // If the '4' key is pressed

            // Read 20 temperature values at ~2 per second
            printf("\n");
            printf("Reading 20 Temperature Values...\n");
            for(i=0;i<20;i++){
                stat = tsys02d_read_temperature(&temperature);
                if(stat==tsys02d_status_ok){
                    printf("%2d: %5.2f%cC",i+1,temperature,248);
                }else if(stat==tsys02d_status_i2c_transfer_error){
                    printf("Transfer Error.");
                }else if(stat==tsys02d_status_crc_error){
                	printf("CRC Error.");
                }
                printf("\n");
                usleep( (500-TSYS02D_CONV_DELAY_MS)*1000 );
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys02d_main_menu();

        }else if(key_input == 27){    // If the 'ESC' key is pressed

            // Print done and exit.
            printf("Done.\n");
            break;

        }else{                      // If some other key is pressed

            // Redisplay the main menu
            tsys02d_main_menu();

        }

    }

    return 0;

}

void tsys02d_main_menu(void){

    //Clear the screen
    printf("\033[2J");

    //Display the main menu
    printf("*******************************************\n");
    printf("****      Measurement Specialties      ****\n");
    printf("*******************************************\n");

    printf("\n");
    printf("    TSYS02D - Digital Temperature Sensor   \n");
    printf("-------------------------------------------\n");

    printf("\n");
    printf("Select a task:\n");
    printf("  (1)   - Reset\n");
    printf("  (2)   - Read Serial Number\n");
    printf("  (3)   - Read Temperature Once\n");
    printf("  (4)   - Read Temperature 20 Times\n");
    printf(" (ESC)  - Quit\n");
    printf("\n");

    return;
}
