/*
 * tsys02d.c
 *
 *  Created on: Aug 12, 2014
 *      Author: JMBrinkhus
 */

#include "tsys02d.h"
#include "xiic_l.h"
#include "sleep.h"
#include "math.h"
#include "xparameters.h"

// TSYS02D Global Variables
u32 			tsys02d_axi_address;
u8 				tsys02d_serial_number[8];

/*********************************************************************
 *
 * Function:    tsys02d_init
 *
 * Description:	Initialize the axi address of the i2c core
 *
 * Parameters:	u32 axi_address -
 * 					Address of Xilinx AXI IIC Peripheral
 *
 * Returns:		void
 *
 *********************************************************************/
void tsys02d_init(u32 axi_address){

	tsys02d_axi_address = axi_address;

	return;

}

/*********************************************************************
 *
 * Function:    tsys02d_reset
 *
 * Description:	Send I2C Reset command to TSYS02D and wait 100ms
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type tsys02d_status with possible values
 * 				tsys02d_status_ok or tsys02d_status_i2c_transfer_error
 *
 *********************************************************************/
tsys02d_status tsys02d_reset(void){

	char tx_buf[1];
	int byte_count=0;

	tx_buf[0] = TSYS02D_I2C_CMD_RESET;
	byte_count = XIic_Send(tsys02d_axi_address, TSYS02D_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
	if(byte_count!=1){
		return tsys02d_status_i2c_transfer_error;
	}
	//Device requires time to complete reset operation
	usleep(100000);

	return tsys02d_status_ok;

}

/*********************************************************************
 *
 * Function:    tsys02d_read_serial_number
 *
 * Description:	Read serial number values from PROM and perform CRC
 * 				check on the data for accuracy
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type tsys02d_status with possible values
 * 				tsys02d_status_ok or tsys02d_status_i2c_transfer_error
 * 				or tsys02d_status_crc_error
 *
 *********************************************************************/
tsys02d_status tsys02d_read_serial_number(void){

	char tx_buf[2];
	char rx_buf[8];
	int i;
	int byte_count=0;

	//Read the first 8 bytes of serial number data
	tx_buf[0] = (TSYS02D_I2C_CMD_READ_PROM_FIRST8 >> 8);
	tx_buf[1] = (TSYS02D_I2C_CMD_READ_PROM_FIRST8 &  0xFF);
	byte_count = XIic_Send(tsys02d_axi_address, TSYS02D_I2C_ADDR, (u8*)tx_buf, 2, XIIC_STOP);
	if(byte_count!=2){
		return tsys02d_status_i2c_transfer_error;
	}
	byte_count = XIic_Recv(tsys02d_axi_address, TSYS02D_I2C_ADDR, (u8*)rx_buf, 8, XIIC_STOP);
	if(byte_count!=8){
		return tsys02d_status_i2c_transfer_error;
	}

	//CRC Check the first 8 bytes and store the checked data in tsys02d_serial_number[]
	for(i=0; i<4; i++){
		if( CRC8(&(rx_buf[2*i])) == TRUE ){
			tsys02d_serial_number[i]=rx_buf[2*i];
		}else{
			return tsys02d_status_crc_error;
		}
	}

	//Read the last 6 bytes of serial number data
	tx_buf[0] = (TSYS02D_I2C_CMD_READ_PROM_LAST6 >> 8);
	tx_buf[1] = (TSYS02D_I2C_CMD_READ_PROM_LAST6 &  0xFF);
	byte_count = XIic_Send(tsys02d_axi_address, TSYS02D_I2C_ADDR, (u8*)tx_buf, 2, XIIC_STOP);
	if(byte_count!=2){
		return tsys02d_status_i2c_transfer_error;
	}
	byte_count = XIic_Recv(tsys02d_axi_address, TSYS02D_I2C_ADDR, (u8*)rx_buf, 6, XIIC_STOP);
	if(byte_count!=6){
		return tsys02d_status_i2c_transfer_error;
	}

	//CRC Check the last 6 bytes and store the checked data in tsys02d_serial_number[]
	for(i=0; i<2; i++){
		if( CRC16(&(rx_buf[3*i])) == TRUE ){
			tsys02d_serial_number[2*i+4]=rx_buf[3*i];
			tsys02d_serial_number[2*i+5]=rx_buf[3*i+1];
		}else{
			return tsys02d_status_crc_error;
		}
	}

	return tsys02d_status_ok;

}

/*********************************************************************
 *
 * Function:    tsys02d_read_temperature
 *
 * Description:	Start an ADC conversion, get the result, and compute
 * 				the measured temperature in degrees Celsius
 *
 * Parameters:	float* temperature -
 * 					Pointer to temperature variable.  This function
 * 					writes the temperature into this location.
 *
 * Returns:		Temperature in degrees Celsius
 *
 *********************************************************************/
tsys02d_status tsys02d_read_temperature(float* temperature){

	char tx_buf[1];
	char rx_buf[3];
	u16 adc16;
	int byte_count=0;

	// Send the i2c command to start an ADC conversion
	tx_buf[0] = TSYS02D_I2C_CMD_START_ADC;
	byte_count = XIic_Send(tsys02d_axi_address, TSYS02D_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
	// Verify the command sent successfully or return an error
	if(byte_count!=1){
		return tsys02d_status_i2c_transfer_error;
	}

	// Wait conversion time delay (TSYS02D_CONV_DELAY_MS milliseconds) for the ADC conversion to take place
    usleep(TSYS02D_CONV_DELAY_MS*1000);

	// Send the i2c command to read the ADC values
	//tx_buf[0] = TSYS02D_I2C_CMD_READ_ADC;
	//byte_count = XIic_Send(tsys02d_axi_address, TSYS02D_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
	// Verify the command sent successfully or return an error
	//if(byte_count!=1){
	//	return tsys02d_status_i2c_transfer_error;
	//}

	// Read the ADC values from the i2c bus
	byte_count = XIic_Recv(tsys02d_axi_address, TSYS02D_I2C_ADDR, (u8*)rx_buf, 3, XIIC_STOP);
	// Verify three bytes were read successfully or return an error
	if(byte_count!=3){
		return tsys02d_status_i2c_transfer_error;
	}

	if(CRC16(rx_buf)==TRUE){
		// Concatenate the received bytes into the 16 bit result
		adc16 = 256*rx_buf[0] + rx_buf[1];
		// Use formula to convert ADC result to degrees Celcius
		*temperature  = (float)adc16 * pow(2, -16) * 175.72 - 46.85;
	} else {
		return tsys02d_status_crc_error;
	}

	return tsys02d_status_ok;
}

/*********************************************************************
 *
 * Function:    CRC8
 *
 * Description:	Check 1 byte of data with 8 bits of CRC information.
 * 				Use polynomial X^8 + X^5 + X^4 + 1
 * 				For use with TSYS02D serial number read
 *
 * Parameters:	char* data -
 * 					2 byte character array. data[0] is the data byte
 * 					and data[1] contains the CRC information
 *
 * Returns:		TRUE if operation succeeded else FALSE
 *
 *********************************************************************/
int CRC8(char* data){
	u32 div, poly;
	int i;
	div = 256*data[0]+data[1];
	poly = CRC_POLY;
	//printf("Divisor: 0x%X\n",(unsigned int)div);
	for(i=0;i<8;i++){
		if( (1<<(15-i))&div ){
			//printf("         0x%X\n         0x%X\n",(unsigned int)(poly<<(7-i)),(unsigned int)div);
			div ^= (poly<<(7-i));
		}
	}
	//printf("Result: 0x%X",(unsigned int)div);
	if( (div&(0xFF))==0x00 ){
		return TRUE;
	}else{
		return FALSE;
	}
}

/*********************************************************************
 *
 * Function:    CRC16
 *
 * Description:	Check 2 bytes of data with 8 bits of CRC information.
 * 				Use polynomial X^8 + X^5 + X^4 + 1
 * 				For use with TSYS02D serial number read and checking
 * 				ADC results for several sensors
 *
 * Parameters:	char* data -
 * 					3 byte character array. data[0] is the most sig-
 * 					nificant byte, data[1] is the least significant
 * 					byte, and data[2] contains the CRC information
 *
 * Returns:		TRUE if operation succeeded else FALSE
 *
 *********************************************************************/
int CRC16(char* data){
	u32 div, poly;
	int i;
	div = 256*256*data[0]+256*data[1]+data[2];
	poly = CRC_POLY;
	//printf("Divisor: 0x%X\n",(unsigned int)div);
	for(i=0;i<16;i++){
		if( (1<<(23-i))&div ){
			//printf("         0x%X\n         0x%X\n",(unsigned int)(poly<<(15-i)),(unsigned int)div);
			div ^= (poly<<(15-i));
		}
	}
	//printf("Result: 0x%X",(unsigned int)div);
	if( (div&(0xFF))==0x00 ){
		return TRUE;
	}else{
		return FALSE;
	}
}
